"use strict";

exports.__esModule = true;
exports.default = buildConfigChain;

var _index = require("../index");

var babel = _interopRequireWildcard(_index);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _micromatch = require("micromatch");

var _micromatch2 = _interopRequireDefault(_micromatch);

var _files = require("./loading/files");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function buildConfigChain() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var filename = opts.filename ? _path2.default.resolve(opts.filename) : null;
  var builder = new ConfigChainBuilder(filename);

  try {
    builder.mergeConfig({
      type: "arguments",
      options: opts,
      alias: "base",
      dirname: process.cwd()
    });

    if (opts.babelrc !== false) {
      builder.findConfigs(filename);
    }
  } catch (e) {
    if (e.code !== "BABEL_IGNORED_FILE") throw e;

    return null;
  }

  return builder.configs.reverse();
}

var ConfigChainBuilder = function () {
  function ConfigChainBuilder(filename) {
    _classCallCheck(this, ConfigChainBuilder);

    this.configs = [];
    this.filename = filename;
    this.possibleDirs = null;
  }

  ConfigChainBuilder.prototype.shouldIgnore = function shouldIgnore(ignore, only, dirname) {
    if (!this.filename) return false;

    if (ignore) {
      if (!Array.isArray(ignore)) {
        throw new Error(".ignore should be an array, was " + JSON.stringify(ignore));
      }

      for (var _iterator = ignore, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var pattern = _ref;

        if (this.matchesPattern(pattern, dirname)) return true;
      }
    }

    if (only) {
      if (!Array.isArray(only)) {
        throw new Error(".only should be an array, was " + JSON.stringify(only));
      }

      for (var _iterator2 = only, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray2) {
          if (_i2 >= _iterator2.length) break;
          _ref2 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) break;
          _ref2 = _i2.value;
        }

        var _pattern = _ref2;

        if (this.matchesPattern(_pattern, dirname)) return false;
      }
      return true;
    }

    return false;
  };

  ConfigChainBuilder.prototype.matchesPattern = function matchesPattern(pattern, dirname) {
    if (typeof pattern === "string") {
      if (!this.possibleDirs) {
        this.possibleDirs = [];

        if (this.filename) {
          this.possibleDirs.push(this.filename);

          var current = this.filename;
          while (true) {
            var previous = current;
            current = _path2.default.dirname(current);
            if (previous === current) break;

            this.possibleDirs.push(current);
          }
        }
      }

      return this.possibleDirs.some(_micromatch2.default.filter(_path2.default.resolve(dirname, pattern), {
        nocase: true,
        nonegate: true
      }));
    } else if (typeof pattern === "function") {
      return pattern(this.filename);
    } else {
      return pattern.test(this.filename);
    }
  };

  ConfigChainBuilder.prototype.findConfigs = function findConfigs(loc) {
    var _this = this;

    if (!loc) return;

    if (!_path2.default.isAbsolute(loc)) {
      loc = _path2.default.join(process.cwd(), loc);
    }

    (0, _files.findConfigs)(_path2.default.dirname(loc)).forEach(function (_ref3) {
      var filepath = _ref3.filepath,
          dirname = _ref3.dirname,
          options = _ref3.options;

      _this.mergeConfig({
        type: "options",
        options: options,
        alias: filepath,
        dirname: dirname
      });
    });
  };

  ConfigChainBuilder.prototype.mergeConfig = function mergeConfig(_ref4) {
    var type = _ref4.type,
        options = _ref4.options,
        alias = _ref4.alias,
        loc = _ref4.loc,
        dirname = _ref4.dirname;

    if (!options) {
      return false;
    }

    if (this.filename && this.shouldIgnore(options.ignore, options.only, dirname)) {
      throw Object.assign(new Error("This file has been ignored."), { code: "BABEL_IGNORED_FILE" });
    }

    options = Object.assign({}, options);

    loc = loc || alias;

    var envKey = babel.getEnv();
    if (options.env) {
      var envOpts = options.env[envKey];
      delete options.env;

      this.mergeConfig({
        type: type,
        options: envOpts,
        alias: alias + ".env." + envKey,
        dirname: dirname
      });
    }

    this.configs.push({
      type: type,
      options: options,
      alias: alias,
      loc: loc,
      dirname: dirname
    });

    if (options.extends) {
      var extendsConfig = (0, _files.loadConfig)(options.extends, dirname);

      var existingConfig = this.configs.some(function (config) {
        return config.alias === extendsConfig.filepath;
      });
      if (!existingConfig) {
        this.mergeConfig({
          type: "options",
          alias: extendsConfig.filepath,
          options: extendsConfig.options,
          dirname: extendsConfig.dirname
        });
      }
      delete options.extends;
    }
  };

  return ConfigChainBuilder;
}();