"use strict";

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.findConfigs = findConfigs;
exports.loadConfig = loadConfig;

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _json = require("json5");

var _json2 = _interopRequireDefault(_json);

var _resolve = require("resolve");

var _resolve2 = _interopRequireDefault(_resolve);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var existsCache = {};
var jsonCache = {};

var BABELRC_FILENAME = ".babelrc";
var BABELRC_JS_FILENAME = ".babelrc.js";
var PACKAGE_FILENAME = "package.json";
var BABELIGNORE_FILENAME = ".babelignore";

function exists(filename) {
  var cached = existsCache[filename];
  if (cached == null) {
    return existsCache[filename] = _fs2.default.existsSync(filename);
  } else {
    return cached;
  }
}

function findConfigs(dirname) {
  var foundConfig = false;
  var foundIgnore = false;

  var confs = [];

  var loc = dirname;
  while (true) {
    if (!foundIgnore) {
      var ignoreLoc = _path2.default.join(loc, BABELIGNORE_FILENAME);
      var ignore = readIgnoreConfig(ignoreLoc);

      if (ignore) {
        confs.push(ignore);
        foundIgnore = true;
      }
    }

    if (!foundConfig) {
      var conf = [BABELRC_FILENAME, BABELRC_JS_FILENAME, PACKAGE_FILENAME].reduce(function (previousConfig, name) {
        var filepath = _path2.default.join(loc, name);
        var config = readConfig(filepath);

        if (config && previousConfig) {
          throw new Error("Multiple configuration files found. Please remove one:\n- " + _path2.default.basename(previousConfig.filepath) + "\n- " + name + "\nfrom " + loc);
        }

        return config || previousConfig;
      }, null);

      if (conf) {
        confs.push(conf);
        foundConfig = true;
      }
    }

    if (foundIgnore && foundConfig) break;

    if (loc === _path2.default.dirname(loc)) break;

    loc = _path2.default.dirname(loc);
  }

  return confs;
}

function loadConfig(name, dirname) {
  var filepath = _resolve2.default.sync(name, { basedir: dirname });

  var conf = readConfig(filepath);
  if (!conf) throw new Error("Config file " + filepath + " contains no configuration data");

  return conf;
}

function readConfig(filepath) {
  return _path2.default.extname(filepath) === ".js" ? readConfigJS(filepath) : readConfigFile(filepath);
}

function readIgnoreConfig(filepath) {
  if (!exists(filepath)) return null;

  var file = _fs2.default.readFileSync(filepath, "utf8");
  var lines = file.split("\n");

  lines = lines.map(function (line) {
    return line.replace(/#(.*?)$/, "").trim();
  }).filter(function (line) {
    return !!line;
  });

  return {
    filepath: filepath,
    dirname: _path2.default.dirname(filepath),
    options: { ignore: lines }
  };
}

function readConfigJS(filepath) {
  if (!exists(filepath)) return null;

  var options = void 0;
  try {
    var configModule = require(filepath);
    options = configModule && configModule.__esModule ? configModule.default || undefined : configModule;
  } catch (err) {
    err.message = filepath + ": Error while loading config - " + err.message;
    throw err;
  }

  if (!options || (typeof options === "undefined" ? "undefined" : _typeof(options)) !== "object" || Array.isArray(options)) {
    throw new Error(filepath + ": Configuration should be an exported JavaScript object.");
  }

  return {
    filepath: filepath,
    dirname: _path2.default.dirname(filepath),
    options: options
  };
}

function readConfigFile(filepath) {
  if (!exists(filepath)) return null;

  var content = _fs2.default.readFileSync(filepath, "utf8");

  var options = void 0;
  if (_path2.default.basename(filepath) === PACKAGE_FILENAME) {
    try {
      var json = jsonCache[content] = jsonCache[content] || JSON.parse(content);

      options = json.babel;
    } catch (err) {
      err.message = filepath + ": Error while parsing JSON - " + err.message;
      throw err;
    }
    if (!options) return null;
  } else {
    try {
      options = jsonCache[content] = jsonCache[content] || _json2.default.parse(content);
    } catch (err) {
      err.message = filepath + ": Error while parsing config - " + err.message;
      throw err;
    }

    if (!options) throw new Error(filepath + ": No config detected");
  }

  if ((typeof options === "undefined" ? "undefined" : _typeof(options)) !== "object") throw new Error(filepath + ": Config returned typeof " + (typeof options === "undefined" ? "undefined" : _typeof(options)));
  if (Array.isArray(options)) throw new Error(filepath + ": Expected config object but found array");

  return {
    filepath: filepath,
    dirname: _path2.default.dirname(filepath),
    options: options
  };
}